@@ -239,36 +239,6 @@ def __run_setup():
     print('done')
     print('succeed')
 
-def install_esptool():
-    if os.path.exists('./platform/mcu/esp32/esptool_py'):
-        return
-    try:
-        print("installing esptool ...", end='')
-        sys.stdout.flush()
-        if os.path.isdir('./tmp') == False:
-            os.mkdir('./tmp')
-        os.chdir('./tmp')
-        esptool_url = 'http://alios-things-public.oss-cn-hangzhou.aliyuncs.com/aos-esptool.zip'
-        if download_url(esptool_url, 'esptool.zip', retry=3, printlog=False) == False:
-            print("failed, can not download esptool.zip")
-            return
-        if which('unzip') != None:
-            popen('unzip -qo esptool.zip', shell=True)
-        else:
-            zfobj = zipfile.ZipFile('esptool.zip')
-            zfobj.extractall()
-            zfobj.close()
-        name = 'aos-esptool'
-        shutil.move(name + '/esptool_py', '../platform/mcu/esp32/esptool_py')
-        shutil.rmtree(name)
-        os.remove('esptool.zip')
-        os.chdir('../')
-        print("succeed")
-    except:
-        traceback.print_exc()
-        os.chdir('../')
-        print("failed")
-
 def _run_make(arg_list):
     # check operating system
     host_os = get_host_os()
@@ -281,10 +251,6 @@ def _run_make(arg_list):
     if ret != 'success':
         return
 
-    #install esptool if necessary
-    if '@esp32devkitc' in ' '.join(arg_list):
-        install_esptool()
-
     make_cmds = {
         'Win32': 'cmd/win32/make.exe',
         'Linux64': 'cmd/linux64/make',
@@ -479,6 +445,13 @@ def new(name, scm='git', component=False, protocol=None):
     if d_type == 'program':
         # This helps sub-commands to display relative paths to the created program
         # p.set_root()
+        import requests
+        github_rtt = requests.get('https://raw.githubusercontent.com/alibaba/AliOS-Things/master/README.md').elapsed.total_seconds()
+        gitee_rtt = requests.get('https://gitee.com/alios-things/AliOS-Things/raw/master/README.md').elapsed.total_seconds()
+        if github_rtt < gitee_rtt:
+            aos_os_url = 'https://github.com/alibaba/AliOS-Things.git'
+        else:
+            aos_os_url = 'https://gitee.com/alios-things/AliOS-Things.git'
         url = aos_os_url + '#master'
         d = 'aos'
         aos_sdk_path = Global().get_cfg(AOS_SDK_PATH)
@@ -1265,17 +1238,25 @@ def make():
     if os.path.isfile(os.path.join(os.getcwd(), Cfg.file)):
         pd = Program(os.getcwd())
         os_path = pd.get_cfg(OS_PATH)
-        out_path = os.path.join(os.getcwd(), 'out').replace(os.path.sep, '/')
-        build_dir = 'BUILD_DIR=' + out_path
+
+        build_dir = ''
+        out_path = ''
+        is_build_dir = 'BUILD_DIR=' in make_args or os.environ.get('BUILD_DIR')
+        if not is_build_dir:
+            out_path = os.path.join(os.getcwd(), 'out').replace(os.path.sep, '/')
+            build_dir = 'BUILD_DIR=' + out_path
+
         cube_makefile = 'CUBE_MAKEFILE=' + os.path.join(pd.get_cfg(PROGRAM_PATH), CUBE_MAKEFILE).replace(os.path.sep, '/')
 
         with cd(os_path):
             cube_modify = pd.get_cfg(CUBE_MODIFY)
             if cube_modify == '1':
-                shutil.rmtree(out_path, True)
+                _run_make(['-e', '-f build/Makefile', 'clean'])
+                if build_dir:
+                    shutil.rmtree(out_path, True)
                 pd.set_cfg(CUBE_MODIFY, '0')
 
-            _run_make(['-e', '-f build/Makefile', make_args, build_dir, cube_makefile])
+            _run_make(['-e', '-f build/Makefile', make_args, cube_makefile, build_dir])
     else:
         # aos source code
         _run_make(['-e', '-f build/Makefile', make_args])
@@ -1412,6 +1393,58 @@ def config_(var=None, value=None, global_cfg=False, unset=False, list_config=Fal
     else:
         subcommands['config'].error("too few arguments")
 
+# upload command
+@subcommand('upload',
+            help='upload aos image',
+            description="Upload aos image to target platform.")
+def upload():
+    """ FIXME we only support stlink tools for now """
+    target = ' '.join(sys.argv[2:])
+    log("target: %s\n" % target)
+
+    original_dir = os.getcwd()
+    ret = cd_aos_root()
+    if ret != 'success':
+        return
+
+    image_path = os.path.join(os.getcwd(), 'out', target, 'binary', target + '.bin')
+    log("image_path: %s\n" % image_path)
+
+    host_os = get_host_os()
+    if host_os == None:
+        error('Unsupported Operating System!')
+
+    upload_cmds = {
+        'Win32': 'win32/st-flash.exe',
+        'Linux64': 'linux64/st-flash',
+        'Linux32': 'linux32/st-flash',
+        'OSX': 'osx/st-flash'
+    }
+
+    cmds_dir = os.path.join(os.getcwd(), 'build/cmd').replace('\\', '/')
+    upload_cmd = os.path.join(cmds_dir, upload_cmds[host_os])
+    log('upload_cmd: %s\n' % upload_cmd)
+    addr = '0x08000000'
+
+    erase_cmd = [upload_cmd, 'erase'];
+
+    ret = subprocess.call(erase_cmd, stdout=sys.stdout, stderr=sys.stderr)
+    if ret == 0:
+        print('erase succeed')
+    else:
+        print('erase failed')
+
+    exec_cmd = [upload_cmd, '--reset', 'write', image_path, addr];
+
+    ret = subprocess.call(exec_cmd, stdout=sys.stdout, stderr=sys.stderr)
+    if ret == 0:
+        print('firmware upload succeed')
+    else:
+        print('firmware upload failed')
+
+    if os.path.isdir(original_dir):
+        os.chdir(original_dir)
+
 
 @subcommand('help',
             help='This help screen')
